
(require-extension syntax-case)
(require-extension stx-engine-support)

;; $Id: stx-engine.scm,v 1.9502 2004/01/22 01:11:45 kl Exp kl $

;=============================================================================
; Syntactic sugar 

; This macro emulates <xsl:apply-template> of XSLT
; "select" must be a node-set
; If omitted, (sxml:content current-node) is used.  
(define-macro (xsl:apply-templates . select)
  (cond  
    ((null? select)
     `(stx:apply-templates (sxml:content current-node) 
			   stx:templates current-root $))
    ((null? (cdr select))
    `(stx:apply-templates ,(car select) stx:templates current-root $))
    (else (error "Invalid parameters for 'xsl:apply-templates': " select))))

;------------------------------------------------------------------------------
; These macros provide support for abbreviated stylesheets:
;
;  <Stylesheet> ::= (stx:stylesheet <Template>+)
;  <Template>   ::= (match <SXPath> <Handler>)
;  <SXPath>     ::= SXPath expression
;  <Handler>    ::= (lambda (current-node stx:templates current-root $) ...)
; 
; For example:
;  (stx:stylesheet
;    (match "//element[state/@condition='standard']"
;	   (lambda (current-node stx:templates current-root $)
;	     (sxml:text current-node)))
;    (match (table (tr 4))
;	   (lambda (current-node stx:templates current-root $)
;	     `(ol
;		,@(map 
;		    (lambda(x) `(li ,x))
;		,((sxpath '(td *text*)) current-node))))))

(define-syntax  sxml:stylesheet
   (syntax-rules  ()
 		 ((stx rule ...)
		  (list 
		    ; default handler
		    (list '*default* 
			  (lambda (node bindings root environment)
			     (stx:apply-templates (sxml:content node) 
						  bindings 
						  root environment)
			     ))
		    ; handler for textual nodes
		    (list '*text* 
			  (lambda(text) text)) 
		    rule ...))))

(define-syntax  sxml:match
   (syntax-rules  ()
 		 ((sxml:match pattern handler)
		   (list (cond ((symbol? pattern) pattern )
			       (else (sxpath pattern)))
			 handler))
 		 ((sxml:match pattern ns-bindings handler)
		   (list (cond ((symbol? pattern) pattern )
			       (else (sxpath pattern ns-bindings)))
			 handler))
		 ))
